--------------------------------------------------------------------------------
-- [VHDL Praktikum SS2004]
--
-- Aufgabe 3:     Minima-Rechenwerk
-- Entity:        E_MINIMA (Top-Level-Entity)               
--
-- Andreas Schibilla (ii4900)
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;


--------------------------------------------------------------------------------
-- Minima-Entity    
--------------------------------------------------------------------------------
entity E_MINIMA is    
    port (
        CLK:         in  std_logic;
        RST:         in  std_logic;
        
        --SR_OUT:      out std_logic_vector(19 downto 0);
        ACCU_OUT:    out std_logic_vector(19 downto 0);
                     
        -- zum Debuggen                              
        A_IMPULS:    out std_logic;
        B_IMPULS:    out std_logic;
        
        COUNTER_OUT: out std_logic_vector(4 downto 0)
        --D_RAM:       out std_logic_vector(19 downto 0);
        --D_S_RAM:     out std_logic
    );
end E_MINIMA;


--------------------------------------------------------------------------------
-- Minima-Architecture
-- fasst alle Komponenten zusammen und reprsentiert die Maschine nach auen
--------------------------------------------------------------------------------
architecture A_MINIMA of E_MINIMA is
-- Komponenten: 
component E_IMPULSE is    
    port (
        CLK:         in  std_logic;
        RST:         in  std_logic; 
        A_IMPULS:    out std_logic;
        B_IMPULS:    out std_logic;
        C_IMPULS:    out std_logic;
        CNT_OUT:     out STD_LOGIC_VECTOR (4 DOWNTO 0)
    );
end component;

component E_SPEICHER is    
    port (
        CLK:         in  std_logic; 
        RST:         in  std_logic;
        ACCU_S_IN:   in  std_logic;
        BZ_S_IN:     in  std_logic;
        F:           in  std_logic;
        U:           in  std_logic;
        A_IMPULS:    in  std_logic;
        B_IMPULS:    in  std_logic;
        ADDR_BUS:    in  std_logic_vector(7 downto 0);
        
        S_OUT:       out std_logic;                      
        -- temp.
        Q_OUT        : OUT STD_LOGIC_VECTOR (19 DOWNTO 0)
    );
end component;

component E_STEUERWERK is    
    port (
        CLK:         in  std_logic;
        RST:         in  std_logic;
        A_IMPULS:    in  std_logic;
        B_IMPULS:    in  std_logic;
        ACCUBIT_IN:  in  std_logic;
        RAM_IN:      in  std_logic;
        
        E_OUT:       out std_logic;
        F_OUT:       out std_logic;
        P_OUT:       out std_logic;
        Q_OUT:       out std_logic;
        N_OUT:       out std_logic;
        R_OUT:       out std_logic;
        L_OUT:       out std_logic;
        C_OUT:       out std_logic;
        A_OUT:       out std_logic;
        S_OUT:       out std_logic;
        U_OUT:       out std_logic;
        H_OUT:       out std_logic;
        ADDR_OUT:    out std_logic_vector (7 downto 0);
        
        BZ_OUT:      out std_logic;
        BR_OUT:      out std_logic
    );
end component;

component E_ALU is    
    port (
        CLK:         in  std_logic;
        RST:         in  std_logic;
        A_IMPULS:    in  std_logic;
        B_IMPULS:    in  std_logic;
        C_IMPULS:    in  std_logic;
        
        A_IN:        in  std_logic;
        S_IN:        in  std_logic;
        C_IN:        in  std_logic;
        L_IN:        in  std_logic;
        R_IN:        in  std_logic;
        N_IN:        in  std_logic;
        
        BR_IN:       in  std_logic;
        RAM_IN:      in  std_logic;
        
        ACCU_S_OUT:  out std_logic;
        ACCU_Q_OUT:  out std_logic_vector(19 downto 0);
                     
        -- zum Debuggen
        DE_AS_OUT:   out std_logic;
        DE_SH_OUT:   out std_logic;
        DE_ADD_OUT:  out std_logic
    );
end component;

signal INT_CLK          : std_logic;
signal INT_HALT         : std_logic;
signal INT_A_IMPULS     : std_logic;
signal INT_B_IMPULS     : std_logic;
signal INT_C_IMPULS     : std_logic; 

signal INT_SR           : std_logic_vector(19 downto 0);
signal INT_BZ           : std_logic;
signal INT_BR           : std_logic;
signal INT_ACCU_S       : std_logic;
signal INT_ACCU_Q       : std_logic_vector(19 downto 0);
signal INT_RAM_OUT      : std_logic;

begin      
 -- Komponenten einbinden

 IMPULSE_inst : E_IMPULSE PORT MAP (         --IN--
                                             CLK         => INT_CLK, 
                                             RST         => RST,
                                             --OUT--
                                             A_IMPULS    => INT_A_IMPULS,
                                             B_IMPULS    => INT_B_IMPULS,
                                             C_IMPULS    => INT_C_IMPULS,
                                             CNT_OUT     => COUNTER_OUT
                                   );

STEUERWERK_inst : E_STEUERWERK PORT MAP (    --IN--
                                             CLK         => INT_CLK,
                                             RST         => RST,
                                             A_IMPULS    => INT_A_IMPULS,
                                             B_IMPULS    => INT_B_IMPULS,
                                             ACCUBIT_IN  => INT_ACCU_Q(19),
                                             RAM_IN      => INT_RAM_OUT,
                                             --OUT--           
                                             E_OUT       => INT_SR(19),
                                             F_OUT       => INT_SR(18),
                                             P_OUT       => INT_SR(17),
                                             Q_OUT       => INT_SR(16),
                                             N_OUT       => INT_SR(15),
                                             R_OUT       => INT_SR(14),
                                             L_OUT       => INT_SR(13),
                                             C_OUT       => INT_SR(12),
                                             A_OUT       => INT_SR(11),
                                             S_OUT       => INT_SR(10),
                                             U_OUT       => INT_SR(9),
                                             H_OUT       => INT_SR(8),
                                             ADDR_OUT    => INT_SR(7 downto 0),
                                             BZ_OUT      => INT_BZ,
                                             BR_OUT      => INT_BR
                                         );

ALU_inst : E_ALU PORT MAP (                  --IN--
                                             CLK         => INT_CLK,
                                             RST         => RST,
                                             A_IMPULS    => INT_A_IMPULS,
                                             B_IMPULS    => INT_B_IMPULS,
                                             C_IMPULS    => INT_C_IMPULS,
                                             A_IN        => INT_SR(11),
                                             S_IN        => INT_SR(10),
                                             C_IN        => INT_SR(12),
                                             L_IN        => INT_SR(13),
                                             R_IN        => INT_SR(14),
                                             N_IN        => INT_SR(15),
                                             BR_IN       => INT_BR,
                                             RAM_IN      => INT_RAM_OUT,
                                             --OUT--
                                             ACCU_S_OUT  => INT_ACCU_S,
                                             ACCU_Q_OUT  => INT_ACCU_Q
                                             
                                             -- zum Debuggen
                                             --DE_AS_OUT   => ,
                                             --DE_SH_OUT   => ,
                                             --DE_ADD_OUT  =>
                           );
                           
SPEICHER_inst : E_SPEICHER PORT MAP (        --IN--
                                             CLK         => INT_CLK,
                                             RST         => RST,
                                             ACCU_S_IN   => INT_ACCU_S,
                                             BZ_S_IN     => INT_BZ,
                                             F           => INT_SR(18),
                                             U           => INT_SR(9),
                                             A_IMPULS    => INT_A_IMPULS,
                                             B_IMPULS    => INT_B_IMPULS,
                                             ADDR_BUS    => INT_SR(7 downto 0),
                                             --OUT--
                                             S_OUT       => INT_RAM_OUT
                                             -- zum Debuggen
                                             --Q_OUT       => D_RAM
                                    );
    
 -- Signale zuweisen   
 INT_CLK  <= CLK and not INT_HALT;
 A_IMPULS <= INT_A_IMPULS;
 B_IMPULS <= INT_B_IMPULS;
 --SR_OUT   <= INT_SR;
 ACCU_OUT <= INT_ACCU_Q;
 
                           
 -- Prozess der bei Befehlen mit H-Bit die Maschine anhlt
 -- (die Taktleitung unterbricht)
 P1: process(CLK, RST)
 begin
   if RST = '1' then
      INT_HALT <= '0';
      
   elsif rising_edge(CLK) then
      if INT_SR(8) = '1' then
         INT_HALT <= '1';
      end if;
   end if;
 end process P1; 
             
end A_MINIMA;




